/*
 * Copyright Aduna (http://www.aduna-software.com/) (c) 1997-2007.
 *
 * Licensed under the Aduna BSD-style license.
 */

package info.aduna.collections.iterators;

import java.util.Iterator;

import info.aduna.iteration.MinusIteration;

/**
 * An iterator that returns the results of an iterator (the left argument) minus
 * the results of another iterator (the right argument). Optionally, the
 * iterator can be configured to filter duplicates from the returned elements.
 * <p>
 * Note that duplicates can also be filtered by wrapping this iterator in a
 * {@link DistinctIterator}, but that has a bit more overhead as it adds a
 * second hash table lookup.
 */
public class MinusIterator<E> extends MinusIteration<E, RuntimeException> implements CloseableIterator<E> {

	/**
	 * Creates a new MinusIterator that returns the results of the left argument
	 * minus the results of the right argument. By default, duplicates are
	 * <em>not</em> filtered from the results.
	 * 
	 * @param leftArg
	 *        An iterator containing the main set of elements.
	 * @param rightArg
	 *        An iterator containing the set of elements that should be filtered
	 *        from the main set.
	 */
	public MinusIterator(Iterator<? extends E> leftArg, Iterator<? extends E> rightArg) {
		this(leftArg, rightArg, false);
	}

	/**
	 * Creates a new MinusIterator that returns the results of the left argument
	 * minus the results of the right argument.
	 * 
	 * @param leftArg
	 *        An iterator containing the main set of elements.
	 * @param rightArg
	 *        An iterator containing the set of elements that should be filtered
	 *        from the main set.
	 * @param distinct
	 *        Flag indicating whether duplicate elements should be filtered from
	 *        the result.
	 */
	public MinusIterator(Iterator<? extends E> leftArg, Iterator<? extends E> rightArg, boolean distinct) {
		super(new IteratorIteration<E, RuntimeException>(leftArg), new IteratorIteration<E, RuntimeException>(
				rightArg), distinct);
	}
}
